@@ -1,27 +1,38 @@
 #!/usr/local/bin/perl
 
-# contributed from Paul Lindner <lindner@reliefweb.int>
-
-package WWW::Search::Gopher;
-
 =head1 NAME
 
-WWW::Search::Gopher - class for searching Gopher
-
+WWW::Search::Gopher - class for searching Gopher pages
 
 =head1 SYNOPSIS
 
     require WWW::Search;
     $search = new WWW::Search('Gopher');
 
-
 =head1 DESCRIPTION
 
-Not documented.
+This class is a specialization of WWW::Search that searches Gopher
+index items.
+
+This class exports no public interface; all interaction should be done
+through WWW::Search objects.
+
+=head1 AUTHOR
+
+C<WWW::Search::NULL> is written by Paul Lindner, <lindner@itu.int>
+
+=head1 COPYRIGHT
+
+Copyright (c) 1997,98 by the United Nations Administrative Committee 
+on Coordination (ACC)
+
+All rights reserved.
 
 =cut
 
 
+package WWW::Search::Gopher;
+
 require Exporter;
 @EXPORT = qw();
 @EXPORT_OK = qw();
@@ -31,6 +42,9 @@ require Exporter;
 use Carp ();
 require WWW::SearchResult;
 
+use vars qw( $VERSION );
+$VERSION = '2.01';
+
 my($debug) = 0;
 
 sub native_setup_search {
@@ -70,7 +84,7 @@ sub native_retrieve_some
 
     # get some
     my($url, $search) = split(/\?/, $self->{_next_url});
-    my $url = new URI::URL($url);
+    $url = new URI::URL($url);
     &GopenServer($url->host, $url->port);
     my $send = $url->path . "\t$search";
     $send =~s/^..//;
@@ -78,7 +92,6 @@ sub native_retrieve_some
     &Gsend($send);
     alarm(0);
 
-    my($self) = @_;
     my(%srchitem);
     my(@entries);
     
@@ -99,7 +112,6 @@ sub native_retrieve_some
 	$hit->title($title);
 	$hit->score($score);
 	$hit->normalized_score($score);
-	$hit->ref($self->{'search_ref'});
 
 	push(@{$self->{cache}}, $hit);
 		
